//
//  voroMap.cpp
//  voro
//
//  Created by Kristopher Krasnosky on 11/28/13.
//  Copyright (c) 2013 Kristopher Krasnosky. All rights reserved.
//

#include "voroMap.h"

voroMap::voroMap(){
    goalDist= INFINITY;
    startDist=INFINITY;
    
    goal[0]=0;
    goal[1]=0;
    start[1]=0;
    start[0]=0;
}

void voroMap::createMap(float xMin, float xMax, float yMin, float yMax, int divisions){
    mapGridScale=divisions;
    int nx,ny;
    std::vector<node> yVect;
    node defaultNode;
    defaultNode.distanceFromStart=0;
   
    nx=(xMax-xMin)*mapGridScale+1;   // 1 added to compensate for 0 axis
    ny=(yMax-yMin)*mapGridScale+1;
    
    yVect.resize(ny, defaultNode);
    grid.resize(nx, yVect);
    
    mapOrigin[0]=-xMin;
    mapOrigin[1]=-yMin;
    _xMin=xMin;
    _xMax=xMax;
    _yMin=yMin;
    _yMax=yMax;
}


voroMap::node * voroMap::getNode(float x, float y){
    voroMap::node *output=NULL;
    int xIndex,yIndex;
    
    xIndex= ( x+mapOrigin[0] ) * mapGridScale;
    yIndex= ( y+mapOrigin[1] ) * mapGridScale;
    
    output = &grid[xIndex][yIndex];
    
    if(output->location.size()==0){  // if this node does not have a location add one
        output->location.push_back(x);
        output->location.push_back(y);
    }
    return output;
}

void voroMap::addLink(float x1, float y1, float x2, float y2){
    getNode(x1, y1)->links.push_back(getNode(x2, y2));  // link first node to seccond node
    getNode(x2, y2)->links.push_back(getNode(x1, y1));  // link seccond node to first
    
    //  node * test=getNode(x2, y2);  //debug
    
    return;
}

void voroMap::addVoro(float *xValues, float *yValues, int count, float minSeparation){

    
    VoronoiDiagramGenerator vdg;  // create a diagram generator
    
    vdg.generateVoronoi(xValues,yValues,count, _xMin,_xMax,_yMin,_yMax,0);   // generate the diagram
    
	vdg.resetIterator();
    
    
    
    //
    // add the diagram to our map
    //
	float x1,y1,x2,y2;
	while(vdg.getNext(x1,y1,x2,y2))
	{
		printf("Line[{{%f,%f},{%f,%f}}],\n",x1,y1,x2, y2);
        addLink(x1, y1, x2, y2);
        
        // create a vecor from each of our points to the goal so we can measure tehm using magnitude;
        std::vector<float> temp1;
        temp1.push_back(x1-goal[0]);
        temp1.push_back(y1-goal[1]);
        std::vector<float> temp2;
        temp2.push_back(x2-goal[0]);
        temp2.push_back(y2-goal[1]);
        std::vector<float> temp1s;
        temp1s.push_back(x1-start[0]);
        temp1s.push_back(y1-start[1]);
        std::vector<float> temp2s;
        temp2s.push_back(x2-start[0]);
        temp2s.push_back(y2-start[1]);
        
        // we need to check to see if any of these points are closer to the goal than our current guess
        if (magnitude(temp1)<goalDist) {
            goalDist=magnitude(temp1);
            goalNearestNode=getNode(x1, y1);
        }
        if (magnitude(temp1s)<startDist) {
            startDist=magnitude(temp1s);
            startNearestNode=getNode(x1, y1);
        }
        if (magnitude(temp2)<goalDist) {
            goalDist=magnitude(temp1);
            goalNearestNode=getNode(x1, y1);
        }
        if (magnitude(temp2s)<startDist) {
            startDist=magnitude(temp2s);
            startNearestNode=getNode(x2, y2);
        }

	}

}


void voroMap::setStart(float x, float y){
    start[0]=x;
    start[1]=y;
    return;
}

void voroMap::setGoal(float x, float y){
    goal[0]=x;
    goal[1]=y;
    return;
}


float voroMap::magnitude(std::vector<float> vect){
    unsigned int i;
    float output=0;
    for (i=0; i<=vect.size(); i++){
        output=output+vect[i]*vect[i];
    }
    output=std::sqrt(output);
    return output;
}

float voroMap::distance(float x1,float y1, float x2,float y2){
    std::vector<float> temp1;
    temp1.push_back(x1-x2);
    temp1.push_back(y1-y2);
    return magnitude(temp1);
}

float voroMap::distance(node *n1, node *n2){
    return distance(n1->location[0], n1->location[1], n2->location[0], n2->location[1]);
}



